//	PhilChooser.h - File chooser using the Phil Component.
//	--------------------------------------------------------------------------
//	This file is part of 'Phil', a very simple file manager/browser for the
//	JUCE library.
//	Copyright (C) 2005-2006  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------

#ifndef PHILCHOOSER_H_
#define PHILCHOOSER_H_

class Phil;
class StandardPhilToolbar;
class PhilChooserStatusbar;

///	File chooser using the Phil Component.
/*!
	I designed this so it could act as a drop-in replacement for JUCE' usual
	FileChooser - the main point to my creating the Phil Component was so I
	can have a file chooser with JUCE on linux.  Most of the documentation is
	just copied from the FileChooser entry in the JUCE reference...
 */
class PhilChooser : public DialogWindow,
					public ChangeListener
{
  public:
	///	Constructor.
	/*!
		\param dialogBoxTitle A text string to display in the dialog box to
		tell the user what's going on.
		\param initialFileOrDirectory The file or directory that should be
		selected when the dialog box opens. If this parameter is set to
		File::nonexistent, a sensible default directory will be used instead.
		\param filePatternsAllowed A set of file patterns to specify which
		files can be selected - each pattern should be separated by a comma
		or semi-colon, e.g. "*" or "*.jpg;*.gif". An empty string means that
		all files are allowed.

		After creating one of these, use one of the browseFor... methods to
		display it.
	 */
	PhilChooser(const String &dialogBoxTitle,
				const File &initialFileOrDirectory=File::nonexistent,
				const String &filePatternsAllowed=String::empty);
	///	Destructor.
	~PhilChooser();

	///	Shows a dialog box to choose a file to open.
	/*!
		\param extraInfoComponent An optional component to display inside
		the dialog box to show special info about the files that the user
		is browsing.  This is put in the sidebar of the Phil Component.
		\return true if the user selected a file, in which case, use the
		getResult() method to find out what it was. Returns false if they
		cancelled instead.

		This will display the dialog box modally, using an "open file" mode,
		so that it won't allow non-existent files or directories to be chosen.
	 */
	bool browseForFileToOpen(FilePreviewComponent *extraInfoComponent=0);
	///	Shows a dialog box to choose a file to save.
	/*!
		\return true if the user chose a file and pressed 'ok', in which
		case, use the getResult() method to find out what the file was.
		Returns false if they cancelled instead.

		This will display the dialog box modally, using an "save file" mode,
		so it will allow non-existent files to be chosen, but not directories.
	 */
	bool browseForFileToSave();
	///	Shows a dialog box to choose a directory.
	/*!
		\return true if the user chose a directory and pressed 'ok', in which
		case, use the getResult() method to find out what they chose.
		Returns false if they cancelled instead.

		This will display the dialog box modally, using an "open directory"
		mode, so it will only allow directories to be returned, not files.
	 */
	bool browseForDirectory();

	///	Returns the last file that was chosen by one of the browseFor methods.
	/*!
		After calling the appropriate browseFor... method, this method lets
		you find out what file or directory they chose.

		Note that the file returned is only valid if the browse method
		returned true (i.e. if the user pressed 'ok' rather than cancelling).
	 */
	const File getResult() const;
	///	Called when the dialog's close button is pressed.
	void closeButtonPressed();
	///	Called when philComponent's state changes.
	void changeListenerCallback(void *objectThatHasChanged);

	///	Sets the colours for the window and Phil Component.
	void setColours(const Colour& background,
					const Colour& boundary = Colours::lightgrey,
					const Colour& text = Colours::grey,
					const Colour& highlight = Colours::lightblue);

	juce_UseDebuggingNewOperator
  private:
	///	Overridden to set the background colour correctly.
	void paint(Graphics &g);

	///	Fills allowedExtensions with an entry for each extension.
	void setupExtensions(const String& str);

	///	The Phil Component used to display the current directory etc.
	Phil *philComponent;

	///	To display the back, forward, up, and home buttons.
	StandardPhilToolbar *toolbar;
	///	To display the Text Editor and ok, cancel buttons.
	PhilChooserStatusbar *statusbar;
	///	To display the FileBrowserInfoComponent in browseForFileToOpen().
	Component *sidebar;

	///	The return File after the dialog's returned.
	File returnFile;
	///	Whether the user pressed ok or cancel.
	bool returnVal;
	///	If we're supposed to be selecting a directory.
	bool directorySelect;
	///	If we're supposed to be opening a file.
	/*!
		We need this for browseForFileToOpen(), because it wouldn't make sense
		to return a non-existent file, while it wouldn't matter for the other
		two browse methods.
	 */
	bool fileOpen;
};

#endif
